import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * Wombat. A Wombat moves forward until it can't do so anymore, at
 * which point it turns left. If a wombat finds a leaf, it eats it.
 * 
 * @author mlewis
 */
public class Wombat extends Actor {
    private int direction;
    private int leavesEaten;
    
    public Wombat() {
        setDirection(0);
        leavesEaten = 0;
    }

    /**
     * Do whatever the wombat likes to to just now.
     */
    public void act() {
        if(foundLeaf()) {
            eatLeaf();
        }
        else if(canMove()) {
            move();
        }
        else {
            turnLeft();
        }
    }

    /**
     * Check whether there is a leaf in the same cell as we are.
     */
    public boolean foundLeaf() {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        return leaf != null;
    }
    
    /**
     * Eat a leaf.
     */
    public void eatLeaf() {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if(leaf != null) {
            // eat the leaf...
            getWorld().removeObject(leaf);
            leavesEaten = leavesEaten + 1; 
        }
    }
    
    /**
     * Move one cell forward in the current direction.
     */
    public void move() {
        if (canMove()) {
            setLocation(getX()+getXOffset(direction),getY()+getYOffset(direction));
        }
    }

    /**
     * Test if we can move forward. Return true if we can, false otherwise.
     */
    public boolean canMove() {
        World myWorld = getWorld();
        int x = getX()+getXOffset(direction);
        int y = getY()+getYOffset(direction);
        // test for outside border
        if (x >= myWorld.getWidth() || y >= myWorld.getHeight()) {
            return false;
        }
        else if (x < 0 || y < 0) {
            return false;
        }
        return true;
    }

    /**
     * Turns towards the left.
     */
    public void turnLeft() {
        direction=direction-1;
        if(direction<0) {
            direction=3;
        }
        setDirection(direction);
    }

    /**
     * Sets the direction we're facing.
     */
    public void setDirection(int dir) {
        direction = dir;
        setRotation(90*direction);
    }

    /**
     * Tell how many leaves we have eaten.
     */
    public int getLeavesEaten() {
        return leavesEaten;
    }

    /**
     * Returns the x offset for the specified direction.
     * @param dir The direction we want an offset for.
     * @return The x offset.
     */
    public int getXOffset(int dir) {
        if(dir==0) {
            return 1;
        } else if(dir==2) {
            return -1;
        } else {
            return 0;
        }
    }

    /**
     * Returns the y offset for the specified direction.
     * @param dir The direction we want an offset for.
     * @return The y offset.
     */
    public int getYOffset(int dir) {
        if(dir==1) {
            return 1;
        } else if(dir==3) {
            return -1;
        } else {
            return 0;
        }
    }
}